from opti.problems import ZDT1, Hyperellipsoid, Sphere

from mbo.algorithm import RandomSearch


def test_unconstrained_problem():
    problem = ZDT1(n_inputs=5)
    optimizer = RandomSearch(problem)

    X = optimizer.propose(n_proposals=10)
    assert len(X) == 10
    assert problem.inputs.contains(X).all()

    optimizer.run(n_proposals=3, n_steps=10)
    assert len(optimizer.problem.get_data()) == 30

    config = optimizer.to_config()
    assert config["method"] == "RandomSearch"


def test_constrained_problem():
    problem = Hyperellipsoid()
    optimizer = RandomSearch(problem)

    X = optimizer.propose(n_proposals=10)
    assert len(X) == 10
    assert problem.inputs.contains(X).all()


def test_single_objective_problem():
    problem = Sphere(n_inputs=3)
    optimizer = RandomSearch(problem)

    X = optimizer.propose(n_proposals=10)
    assert len(X) == 10
    assert problem.inputs.contains(X).all()



from typing import Optional

import pandas as pd
from opti.problem import Problem


class Algorithm:
    """Base class for Bayesian optimization algorithms"""

    def __init__(self, problem: Problem):
        self._problem = problem
        self.model = None
        self._initialize_problem()

    @property
    def problem(self):
        return self._problem

    @problem.setter
    def problem(self, problem):
        self._problem = problem
        self._initialize_problem()

    def _initialize_problem(self) -> None:
        """Problem related initializations or checks."""
        pass

    def _fit_model(self) -> None:
        """Fit a probabilistic model to the available data."""
        pass

    def _tune_model(self) -> None:
        """Tune the model parameters."""
        pass

    def copy(self, data: Optional[pd.DataFrame] = None) -> "Algorithm":
        """Creates a copy of the optimizer where the data is possibly replaced."""
        new_opt = self.from_config(self.to_config())
        if data is not None:
            new_opt.problem.set_data(data)
            new_opt._fit_model()
        return new_opt

    def add_data_and_fit(self, data: pd.DataFrame) -> None:
        """Add new data points and refit the model."""
        self.problem.add_data(data)
        self._fit_model()
        self._tune_model()

    def sample_initial_data(self, n_samples: int):
        """Create an initial data set for problems with known function y=f(x)."""
        self.problem.create_initial_data(n_samples)
        self._fit_model()

    def predict(self, X: pd.DataFrame) -> pd.DataFrame:
        """Evaluate the posterior predictive mean and standard deviation."""
        raise NotImplementedError

    def predict_pareto_front(self, **kwargs) -> pd.DataFrame:
        """Calculate a finite representation the Pareto front of the model."""
        raise NotImplementedError

    def propose(self, n_proposals: int = 1) -> pd.DataFrame:
        """Propose a set of experiments according to the algorithm."""
        raise NotImplementedError

    def run(
        self, n_proposals: int = 1, n_steps: int = 10, show_progress: bool = True
    ) -> None:
        """Run the BO algorithm to optimize the problem."""
        if self.problem.f is None:
            raise ValueError(
                "The problem has no function defined. For external function evaluations use the propose() method instead."
            )

        for _ in range(n_steps):
            X = self.propose(n_proposals)
            Y = self.problem.f(X)
            self.add_data_and_fit(pd.concat([X, Y], axis=1))

    def get_model_parameters(self) -> pd.DataFrame:
        """Get the parameters of the surrogate model."""
        raise NotImplementedError

    def to_config(self) -> dict:
        """Serialize the algorithm settings to a config dict."""
        raise NotImplementedError

    @classmethod
    def from_config(cls, config: dict) -> "Algorithm":
        """Create an algorithm instance from a configuration dict."""
        problem = Problem.from_config(config["problem"])
        parameters = config.get("parameters", {})
        return cls(problem, **parameters)



import opti

from mbo.algorithm import Algorithm


class RandomSearch(Algorithm):
    def __init__(self, problem: opti.Problem):
        """Random selection of new data.

        Args:
            problem: Optimization problem
            data: Initial Data to use instead of the problem.data
        """
        super().__init__(problem)

    def propose(self, n_proposals=1):
        df = self.problem.sample_inputs(n_proposals)
        df.index += 0 if self.problem.data is None else len(self.problem.data)
        return df

    def to_config(self) -> dict:
        return {"method": "RandomSearch", "problem": self._problem.to_config()}



import os.path

from setuptools import find_packages, setup


def get_version():
    here = os.path.abspath(os.path.dirname(__file__))
    fp = os.path.join(here, "mbo/__init__.py")
    for line in open(fp).readlines():
        if line.startswith("__version__"):
            return line.split('"')[1]
    return ""


root_dir = os.path.dirname(__file__)
with open(os.path.join(root_dir, "README.md"), "r") as fh:
    long_description = fh.read()

setup(
    name="mbo",
    description="Multiobjective Bayesian Optimization",
    author="BASF SE",
    license="BSD-3",
    url="https://github.com/basf/mbo",
    keywords=[
        "Bayesian optimization",
        "Multi-objective optimization",
        "Experimental design",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3 :: Only",
        "License :: OSI Approved :: BSD License",
        "Topic :: Scientific/Engineering",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Developers",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
    version=get_version(),
    python_requires=">=3.7",
    packages=find_packages(),
    include_package_data=True,
    install_requires=["mopti>=0.1.0", "numpy", "pandas"],
    extras_require={"testing": ["pytest"]},
)



from opti.problems import ZDT1

from mbo.algorithm import Algorithm


def test_init():
    # set up with initial data from problem
    problem = ZDT1(n_inputs=3)
    problem.create_initial_data(6)
    optimizer = Algorithm(problem)
    assert len(optimizer.problem.get_data()) == 6
